%% TEXT CATALOGING version 2.2: main program
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
/*
AUTHOR: Pascual Julian-Iranzo (Universidad de Castilla-La Mancha, Spain)
OTHER CONTRIBUTORS: Fernando Saenz-Perez (Univ. Complutense de Madrid, Spain)

This software is licensed for research and educational purposes only and it is
distributed with NO WARRANTY OF ANY KIND. You are freely allowed to use, copy
and distribute WN_CONNECT provided that you make no modifications to any of its
files and give credit to their original authors.
*******************************************************************************/

%% Connexion with WordNet
:- wn_connect.

%% For this problem, where we mainly unify words (constans) Sessa's weak unification
%% algorithm (a1) is enough and less expensive  
:- weak_unification(a1). 

%%%%%%%%%%%%%%%%%%%%%
%%% include modules
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

:- include('cataloging_modules/auxiliary.bpl').

%%% genontology.bpl
:- include('cataloging_modules/genontology_modules/englishPlural.bpl').
:- include('cataloging_modules/genontology_modules/etu.bpl').
:- include('cataloging_modules/genontology.bpl').

%%% inspect.bpl
:- include('cataloging_modules/inspect_modules/cd_operators.bpl').
:- include('cataloging_modules/inspect_modules/zip.bpl').
:- include('cataloging_modules/inspect.bpl').

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% welcome_message
%
%     Shows the initial welcome message.
%

welcome_info :-
	nl,
	write('                                                     Universidad de'), nl,
	write('                                                  Castilla - La Mancha'), nl,
	write('[T [E \\X [T - (C /A [T /A [L (O) (G [I [N (G.  (Version 2.1 - March 2020)'), nl,
	write('-----------------------------------------------------------------------------'), nl,
	write(' AUTHOR: Pascual Julian-Iranzo (Universidad de Castilla-La Mancha, Spain)'), nl,
	write(' OTHER CONTRIBUTORS: Fernando Saenz-Perez (Univ. Complutense de Madrid, Spain)'), nl,
	nl,
	write('    This software is licensed for research and educational purposes only and'), nl,
	write('    it is distributed with NO WARRANTY OF ANY KIND. You are freely allowed to'), nl, 
	write('    use, copy and distribute it provided that you make no modifications to any'), nl,
	write('    of its files and give credit to their original authors.'), nl,
	write('-----------------------------------------------------------------------------'), nl,
	write('-----------------------------------------------------------------------------'), nl,
	write('--    Enter \'info\' to get help on some of the available predicates       --'), nl,
	write('-----------------------------------------------------------------------------'), nl,
	nl.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% help predicates
%
%     Show information of some of the available predicates.
%

info :-  
     write('-------------------------------------------------------------------------------'), nl,
     write('[ PREDICATE        ][  DESCRIPTION                                            ]'), nl,
     write('-------------------------------------------------------------------------------'), nl,
     write('-------------------------------------------------------------------------------'), nl,
	write('welcome_info:        Writes a welcome message with license information.        '), nl,
	write('-------------------------------------------------------------------------------'), nl,
	write('-------------------------------------------------------------------------------'), nl,
	write('convert_to_singular: Converts a plural word into singular.                     '), nl,
	write('gen_glossList:       Tokenizes the gloss of a word.                            '), nl,
	write('gen_hyponymList:     Generates a subset of hyponyms from a word.               '), nl,
	write('gen_ontology_file:   Generates an ontology from a list of words which is stored'), nl,
	write('                     into a file.                                              '), nl,
	write('-------------------------------------------------------------------------------'), nl,
	write('-------------------------------------------------------------------------------'), nl,
	write('inspect:             Inspects a file seeking for words which are similar to a  '), nl,
	write('                     Keyword and generates account information.                '), nl, 
	write('seqInspect:          Like inspect but for a list of Keywords.                  '), nl,
	write('th_catalog:          Produces an automatic classification of a dataset.        '), nl,
	write('experiment:          Compares the automatic classification of the dataset with '), nl,
	write('                     an expert classification and generates statistical data.  '), nl,
	write('-------------------------------------------------------------------------------'), nl,
	write('-------------------------------------------------------------------------------'), nl,
	write('--    Enter \'info(Predicate)\' to obtain more help on this predicate         ---'), nl,
	write('-------------------------------------------------------------------------------'), nl,
	nl.


info(gen_ontology_file):-
write('gen_ontology_file(+ListOfCategories, +File, +Measure, +Mode):'), nl, 
write('       Given a list of categories, ListOfCategories, the name of a file, File, and the'), nl, 
write('       acronym of a measure, Measure (by now [path, wup, lch, res, lin, jcn, yarm]), it'), nl, 
write('       generates a set of proximity equations and stores them into the file File.'), nl,
nl, 
write('       If Mode=glosses, the ontology is built starting from the glosses of the words in'), nl, 
write('       ListOfCategories. '), nl,
nl,
write('       If Mode=hyponymy or Mode=hyponymy(Level), the ontology is built from a subset of '), nl, 
write('       the hyponyms of the words in ListOfCategories.'), nl, 
write('       If Mode=hyponymy, only up to two levels of hyponyms are generated.'), nl,
write('       If Mode=hyponymy(N) then N levels of hyponyms are generated.'), nl,
nl,
write('gen_ontology_file(+ListOfCategories, +File, +Measure):'), nl, 
write('       As the predicate above, but the parameter \'Mode\' is set to the value \'glosses\' '), nl,
write('       by default.'), nl,
nl,
write('       EXAMPLES: gen_ontology_file([cat:n:1,lion:n:1], ontFile1, wup, hyponymy).'), nl, 
write('                 gen_ontology_file([tree:n:1,grass:n:1], ontFile2, wup, hyponymy(3)).'), nl, 
write('                 gen_ontology_file([cat:n:1,lion:n:1], ontFile3, wup).'), nl, !.


info(gen_hyponymList):-
write('gen_hyponymList(+Word, +Level, -Hyponym_list): '), nl,
write('       Returns the list of hyponyms, \'Hyponym_list\', from the word Word. \'Level\' is the number of'), nl, 
write('       levels of hyponyms which are generated.'), nl, !.


info(gen_glossList):-
write('gen_glossList(+Word, -Gloss_list): '), nl,
write('       Returns the list of tokens, \'Gloss_list\', from the gloss of the word \'Word\'.'), nl, !.

info(convert_to_singular):-
write('convert_to_singular(+ Word, -SWord): '), nl,
write('       Converts a plural word, \'Word\', into a singular word \'SWord\'.'), nl, !.

info(inspect):-
write('inspect(+FileName, +Keyword, -DataAccount): '), nl,
write('       Inspects a file, named \'FileName\', seeking for words which are similar to '), nl,
write('       a category \'Keyword\'. \'DataAccount\' is an account record forming a list with the '), nl,
write('       following structure:'), nl,
write('           [[texNumber(1)|L1], [texNumber(2)|L2], . . .]'), nl,
write('       There is a sublist account \'L_i\' for each text stored in the file FileName. The list '), nl,
write('       L_i is a list of triples \'t(X,N,D)\'; where X is a term similar to the term Keyword, '), nl,
write('       with degree D, which occurs N times in the text texNumber(i).'), nl,
nl, 
write('       EXAMPLE: inspect(path, \'Files4exps/textos_FP\', wheat, DataAccountList).'), nl, !.

%%%%%%%
info(seqInspect):-
write('seqInspect(+File, +CategoryList, +Process, -ResultList)'), nl,
write('       Sequentially applies the predicate \'inspect\' to the texts of the file \'File\'. It'), nl, 
write('       seeks for words which are similar to the categories in \'CategoryList\' and performs'), nl,
write('       accounting tasks.'), nl,
nl, 
write('       PARAMETERS: '), nl,
write('       \'FileName\': File name or path name of a file containing the documents to be'), nl, 
write('       classified (stored in the the SMART standard format).'), nl,
nl,
write('       \'CategoryList\': is a list of Terms (the so called categories) to be found'), nl,
nl,
write('       \'Process\': A compatibility degree operator (e.g.: sUm, wa, mIn, mAx)'), nl,
nl,
write('       \'ResultList\': A list of elements of type \'r(Category, CategoryResultList)\','), nl,
write('       where \'CategoryResultList\' is a DataAccount List as the one computed by inspect'), nl,
nl,        
write('       EXAMPLE: seqInspect(wup, \'finalexperiments/odp/odp\',[oil:n:3, gas:n:3, '), nl,
write('                  utitily:n:1, electricity:n:2, consulting:v:1, management:n:1, '), nl,
write('                  management:n:2, employment:n:1, employment:n:2, \'fuel cells\':n:1, '), nl,
write('                  association:n:1, association:n:5, hydrogen:n:1], sUm, ResultList).'), nl, !.

%%%%%%%
info(th_catalog):-
write('th_catalog(+Measure, +FileName, +CategoryList, +Process, -TexCatalog): '), nl,
write('       Performs the cataloging task, assigning several categories of the list \'CategoryList\' to '), nl, 
write('       the texts in the file \'FileName\'.'), nl,
nl, 
write('       PARAMETERS: '), nl,
write('       \'Measure\' can be one of the following constants: [ont, path, wup, lch, res, jcn, jin, yarm].'), nl, 
write('       The constant \'ont\' means that you want to perform the experimentes using a predefined ontology,'), nl, 
write('       that you must first load before launching the predicate \'experiment/4\'. The other constants '), nl, 
write('       are the acronyms of standard linguistic similarity measures which are computed thanks to the '), nl, 
write('       connection to the Prolog version of the WordNet database. This connection is automatically '), nl, 
write('       established by the cataloging.bpl program (and it does not require uploading an ontology). '), nl,
nl,
write('       \'FileName\' is the file name or path name of a file containing the documents to be classified '), nl,
write('       (stored in the the SMART standard format).'), nl,
nl,
write('       \'CategoryList\' is a list of categories (i.e., the keywords used to classify the texts).'), nl,
nl, 
write('       \'Process\' is a compatibility degree operator (e.g.: sUm, wa, mIn, mAx) used to perform the'), nl,
write('       computation of the compatibility degree between the keywords and the texts to be classified.'), nl, 
nl,
write('       \'TexCatalog\' is the theoretical classification where different keywords are assigned to the'), nl,
write('       documents to be classified. It is a list with the following structure:'), nl,
write('           [[texNumber(1)|ListOfkeywords1], [texNumber(2)|ListOfkeywords2], ...].'), nl,
nl,
write('       EXAMPLES: th_catalog(wup, \'Files4exps/odp/odp\',[oil:n:3, gas:n:3, utitily:n:1, electricity:n:2, '), nl,
write('                 management:n:1, management:n:2, employment:n:1, employment:n:2, \'fuel cells\':n:1, '), nl,
write('                 consulting:v:1, association:n:1, association:n:5, hydrogen:n:1], sUm, TexCatalog).'), nl,
nl,
write('                 th_catalog(path, \'Files4exps/odp/odp\',[renewable, electricity, oil, gas, utilities,  '), nl,
write('                 \'fuel cells\', hydrogen, consulting, employment, associations, management], sUm, TexCatalog).'), nl, !.




info(experiment):-
nl, 
write('experiment(+Measure, +FileName, +CategoryList, +Process):'), nl,
write('       Performs an experiment comparing the theoretical classification \'TexCatalog\' obtained by the'), nl, 
write('       predicate \'th_catalog\' and the expert classification \'ExpTexCatalog\' associated to the file'), nl,
write('       \'FileName\'. '), nl,
nl,
write('       PARAMETERS:'), nl,
write('       \'Measure\' can be one of the following constants: [ont, path, wup, lch, res, jcn, jin, yarm]. The '), nl,
write('       constant \'ont\' means that you want to perform the experimentes using a predefined ontology, that '), nl,
write('       you must first load before launching the predicate \'experiment/4\'. The other constants are the '), nl,
write('       acronyms of standard linguistic similarity measures which are computed thanks to the connection to '), nl,
write('       the Prolog version of the WordNet database. This connection is automatically established by the '), nl,
write('       cataloging.bpl program (and it does not require uploading an ontology). '), nl,
nl,
write('       \'FileName\' is the file name or path name of a file containing the documents to be classified '), nl,
write('       (stored in the the SMART standard format).'), nl,
nl,
write('       \'CategoryList\' is a list of categories (i.e., the keywords used to classify the texts).'), nl,
nl, 
write('       \'Process\' is a compatibility degree operator (e.g.: sUm, wa, mIn, mAx) used to perform the'), nl,
write('       computation of the compatibility degree between the keywords and the texts to be classified. '), nl,

write('       OUTPUT INFORMATION:'), nl,
write("       Positive:  percentage of positive classifications (In this case, \'Positive\' means that "), nl,
write('                  at least one of the categories assigned by the classifier to a text '), nl,
write('                  is in the set of categories assigned by the expert to that text).'), nl, 
write('       Negative:  percentage of negative classifications; it is a measure of the number'), nl,
write('                  of texts bad classified (Negative means that none of the categories '), nl,
write('                  assigned by the classifier to a text match the expert classification).'), nl,
write('       Undefined: percentage of texts not classified by the classification procedure.'), nl,
nl, 
write('       [OBSERVATION: when computing Positive, Negative, and Undefined, the percentage is w.r.t. '), nl,
write('                    the total number of texts to be classified.]'), nl,

nl, 
write('       Precision: percentage of total positive classifications w.r.t. the total of classifications '), nl,
write('                  performed by the classifier method. In this case \'positive classification\' means'), nl,
write('                  a classification where the classifier and the expert judgement coincides.'), nl,
write('       Recall:   percentage of total positive classifications w.r.t. the total of classifications '), nl,
write('                 performed by the expert classifier.'), nl,
write('       Fmeasure: the harmonic mean between precision and recall.'), nl,
nl, 
write('       EXAMPLE: experiment(ont, \'Files4exps/odp/odp\',[oil:n:3, gas:n:3, utitily:n:1, electricity:n:2, '), nl,
write('                management:n:1, management:n:2, employment:n:1, employment:n:2, \'fuel cells\':n:1, '), nl,
write('                consulting:v:1, association:n:1, association:n:5, hydrogen:n:1], sUm).'), nl.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%:- welcome_info. %% it does not work in BPL



